@@ -1,8 +1,9 @@
 # hooks.py -- for dealing with git hooks
 # Copyright (C) 2012-2013 Jelmer Vernooij and others.
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
-# General Public License as public by the Free Software Foundation; version 2.0
+# General Public License as published by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
 # modify it under the terms of either of these two licenses.
 #
@@ -30,7 +31,7 @@ class Hook:
     """Generic hook object."""
 
     def execute(self, *args):
-        """Execute the hook with the given args
+        """Execute the hook with the given args.
 
         Args:
           args: argument list to hook
@@ -43,7 +44,7 @@ class Hook:
 
 
 class ShellHook(Hook):
-    """Hook by executable file
+    """Hook by executable file.
 
     Implements standard githooks(5) [0]:
 
@@ -58,8 +59,8 @@ class ShellHook(Hook):
         pre_exec_callback=None,
         post_exec_callback=None,
         cwd=None,
-    ):
-        """Setup shell hook definition
+    ) -> None:
+        """Setup shell hook definition.
 
         Args:
           name: name of hook for error messages
@@ -85,13 +86,10 @@ class ShellHook(Hook):
         self.cwd = cwd
 
     def execute(self, *args):
-        """Execute the hook with given args"""
-
+        """Execute the hook with given args."""
         if len(args) != self.numparam:
             raise HookError(
-                "Hook %s executed with wrong number of args. \
-                            Expected %d. Saw %d. args: %s"
-                % (self.name, self.numparam, len(args), args)
+                f"Hook {self.name} executed with wrong number of args. Expected {self.numparam}. Saw {len(args)}. args: {args}"
             )
 
         if self.pre_exec_callback is not None:
@@ -99,44 +97,41 @@ class ShellHook(Hook):
 
         try:
             ret = subprocess.call(
-                [os.path.relpath(self.filepath, self.cwd)] + list(args),
-                cwd=self.cwd)
+                [os.path.relpath(self.filepath, self.cwd), *list(args)], cwd=self.cwd
+            )
             if ret != 0:
                 if self.post_exec_callback is not None:
                     self.post_exec_callback(0, *args)
-                raise HookError(
-                    "Hook %s exited with non-zero status %d" % (self.name, ret)
-                )
+                raise HookError(f"Hook {self.name} exited with non-zero status {ret}")
             if self.post_exec_callback is not None:
                 return self.post_exec_callback(1, *args)
-        except OSError:  # no file. silent failure.
+        except FileNotFoundError:  # no file. silent failure.
             if self.post_exec_callback is not None:
                 self.post_exec_callback(0, *args)
 
 
 class PreCommitShellHook(ShellHook):
-    """pre-commit shell hook"""
+    """pre-commit shell hook."""
 
-    def __init__(self, cwd, controldir):
+    def __init__(self, cwd, controldir) -> None:
         filepath = os.path.join(controldir, "hooks", "pre-commit")
 
         ShellHook.__init__(self, "pre-commit", filepath, 0, cwd=cwd)
 
 
 class PostCommitShellHook(ShellHook):
-    """post-commit shell hook"""
+    """post-commit shell hook."""
 
-    def __init__(self, controldir):
+    def __init__(self, controldir) -> None:
         filepath = os.path.join(controldir, "hooks", "post-commit")
 
         ShellHook.__init__(self, "post-commit", filepath, 0, cwd=controldir)
 
 
 class CommitMsgShellHook(ShellHook):
-    """commit-msg shell hook
-    """
+    """commit-msg shell hook."""
 
-    def __init__(self, controldir):
+    def __init__(self, controldir) -> None:
         filepath = os.path.join(controldir, "hooks", "commit-msg")
 
         def prepare_msg(*args):
@@ -163,9 +158,9 @@ class CommitMsgShellHook(ShellHook):
 
 
 class PostReceiveShellHook(ShellHook):
-    """post-receive shell hook"""
+    """post-receive shell hook."""
 
-    def __init__(self, controldir):
+    def __init__(self, controldir) -> None:
         self.controldir = controldir
         filepath = os.path.join(controldir, "hooks", "post-receive")
         ShellHook.__init__(self, "post-receive", path=filepath, numparam=0)
@@ -195,7 +190,7 @@ class PostReceiveShellHook(ShellHook):
             if (p.returncode != 0) or err_data:
                 err_fmt = b"post-receive exit code: %d\n" + b"stdout:\n%s\nstderr:\n%s"
                 err_msg = err_fmt % (p.returncode, out_data, err_data)
-                raise HookError(err_msg.decode('utf-8', 'backslashreplace'))
+                raise HookError(err_msg.decode("utf-8", "backslashreplace"))
             return out_data
         except OSError as err:
             raise HookError(repr(err)) from err
